#include "mex.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "m_malloc.h"


// See ml_compressHist.m for input/output format
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){   
   /* check for the proper no. of input and outputs */
   if (nrhs != 3)
      mexErrMsgTxt("3 input arguments are required");
   if (nlhs>2)
      mexErrMsgTxt("Too many outputs");


   int rows = mxGetM(prhs[0]);
   int cols = mxGetN(prhs[0]);
   int histSize = mxGetM(prhs[1]);

   int Output = 0;   

   /* Get the pointers to the input Data */  
   double *ImAssignPtr = mxGetPr(prhs[0]);
   double *weightsPtr = mxGetPr(prhs[1]);    
   double thres = (double) (*mxGetPr(prhs[2]));
   
   /* create matrices */
   int *reAssign, nNewBin;
   double *cmprssW;
   
   // compress weights
   cmprssW = buildVector<double>(histSize+1);
   // reassignment
   reAssign = buildVector<int>(histSize+1);
   
   // comrpess weights
   cmprssW[0] = 0;
   nNewBin = 1;   
   double w;
   for (int k=0; k<histSize; k++){
      w = (double) (*weightsPtr);
      if (abs(w) >  thres){
         cmprssW[nNewBin] = w;
         nNewBin++;
         reAssign[k+1] = nNewBin;         
      } else {
         reAssign[k+1] = 1;
      }
      weightsPtr++;
   }

   if (Output){
      printf("rows: %d, cols: %d, histSize: %d, thres: %f, compressed Hist Size: %d \n", 
         rows, cols, histSize, thres, nNewBin);      
   }   

   /* Create the outGoing Array  */
   plhs[0] = mxCreateNumericMatrix(rows, cols, mxDOUBLE_CLASS, mxREAL);   
   double *labelOutPtr = mxGetPr(plhs[0]); 
   
   int oldBin;
   for (int j=0; j<cols; j++){
      for (int i=0; i<rows; i++){
         oldBin = ((int) (*ImAssignPtr));         
         *labelOutPtr = (double) reAssign[oldBin];
         labelOutPtr++;
         ImAssignPtr++;  
      }
   }

   plhs[1] = mxCreateNumericMatrix(nNewBin, 1, mxDOUBLE_CLASS, mxREAL);   
   double *outWPtr = mxGetPr(plhs[1]); 
   for (int k=0; k<nNewBin; k++){
      *outWPtr = (double) cmprssW[k];
      outWPtr++;
   }
   
   // memory release
   destroyVector<double>(cmprssW, histSize+1);
   destroyVector<int>(reAssign, histSize+1);
} 
  