#include "mex.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "m_utils.h"
#include "m_malloc.h"
#include "m_TreeNode1a.h"


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){   
   /* check for the proper no. of input and outputs */
   if (nrhs != 6)
   mexErrMsgTxt("6 input arguments are required");
   if (nlhs>1)
   mexErrMsgTxt("Too many outputs");

   int rows = mxGetM(prhs[0]);
   int cols = mxGetN(prhs[0]);
   int histSize = mxGetM(prhs[1]);

   int Output = 0;

   /* Get the pointers to the input Data */  
   double *ImAssignPtr = mxGetPr(prhs[0]);
   double *targetHistPtr = mxGetPr(prhs[1]);    
   double *weightsPtr = mxGetPr(prhs[2]);
   double *CPtr = mxGetPr(prhs[3]);
   double *tolFactorPtr = mxGetPr(prhs[4]);
   double *knownBestPtr = mxGetPr(prhs[5]);

   double C = (double) *CPtr;
   double tolFactor = (double) *tolFactorPtr;
   double knownBest = (double) *knownBestPtr;

   /* create matrices */
   int **ImAssign, *targetHist;  
   double *weights;

   targetHist = buildVector<int>(histSize);
   weights = buildVector<double>(histSize);
   ImAssign = buildMatrix<int>(rows, cols);

   // Assign the data targetHist
   for (int k=0; k<histSize; k++){
      targetHist[k] = (int) (*targetHistPtr);
      targetHistPtr++;
      weights[k] = (double) (*weightsPtr);
      weightsPtr++;
   }

   // Assign the data for: Im1Assign Im1Init Label1
   for (int j=0; j<cols; j++){
      for (int i=0; i<rows; i++){
         ImAssign[i][j] = ((int) (*ImAssignPtr));
         ImAssignPtr++;  
      }
   }

   if (Output){
      printf("rows: %d, cols: %d, histSize: %d, C: %g \n", rows, cols, histSize, C);
   }
   Window bestBB = findBox1a(ImAssign, rows, cols, targetHist, weights, histSize, 
      C, tolFactor, knownBest);


   /* Create the outGoing Array  */
   plhs[0] = mxCreateNumericMatrix(1, 4, mxDOUBLE_CLASS, mxREAL);   
   double *labelOutPtr = mxGetPr(plhs[0]); 
   
   labelOutPtr[0] = (double) (bestBB.ul_x + 1);
   labelOutPtr[1] = (double) (bestBB.ul_y + 1);
   labelOutPtr[2] = (double) (bestBB.lr_x + 1);
   labelOutPtr[3] = (double) (bestBB.lr_y + 1);
   destroyMatrix<int>(ImAssign, rows, cols);
   destroyVector<int>(targetHist, histSize);   
   destroyVector<double>(weights, histSize);
}
