#include "mex.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "m_utils.h"
#include "m_malloc.h"
#include "m_TreeNode1e.h"
#include "m_TreeNode1b.h"


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){
   /* check for the proper no. of input and outputs */
   if (nrhs > 6)
      mexErrMsgTxt("4 or 5 input arguments are required");
   if (nlhs>2)
      mexErrMsgTxt("Too many outputs");


   int nKeyPnt = mxGetM(prhs[0]);
   int rows = (int) *mxGetPr(prhs[1]);
   int cols = (int) *mxGetPr(prhs[2]);

   int histSize = mxGetM(prhs[3]);

   int Output = 0;

   /* Get the pointers to the input Data */
   double *ImAssignPtr = mxGetPr(prhs[0]);
   double *weightsPtr = mxGetPr(prhs[3]);

   /* create matrices */
   int **ImAssign;
   double *weights;

   weights = buildVector<double>(histSize);
   ImAssign = buildMatrix<int>(nKeyPnt, 3);

   // Assign the data targetHist
   for (int k=0; k<histSize; k++){
      weights[k] = (double) (*weightsPtr);
      weightsPtr++;
   }

   // Assign the data for: Im1Assign Im1Init Label1
   for (int j=0; j<3; j++){
      for (int i=0; i< nKeyPnt; i++){
         ImAssign[i][j] = ((int) (*ImAssignPtr));
         ImAssignPtr++;
      }
   }

   if (Output){
      printf("rows: %d, cols: %d, histSize: %d, nKeyPnt: %d \n",
              rows, cols, histSize, nKeyPnt);
   }

   Window bestBB;
   if (nrhs == 4){
      bestBB = findBox1b(ImAssign, nKeyPnt, rows, cols, weights, histSize);
   } else if (nrhs == 5){
      double *szRngPtr = mxGetPr(prhs[4]);
      int *szRng = buildVector<int>(4);

      for (int k=0; k < 4; k++){
         szRng[k] = (int) (*szRngPtr);
         szRngPtr++;
      }
      SizeRange szRange(szRng[2], szRng[3], szRng[0], szRng[1]);
      if (Output){
         printf("hmin: %d, hmax: %d, wmin: %d, wmax: %d\n",
            szRange.hmin, szRange.hmax, szRange.wmin, szRange.wmax);
      }
      bestBB = findBox1e(ImAssign, nKeyPnt, rows, cols, weights, histSize, szRange);
      destroyVector<int>(szRng, 4);
   }

   /*if (!isImAssignGood(ImAssign, rows, cols)){
      mexErrMsgTxt("ImAssign, 1st agrument must be matrix of positive integers");
   }*/

   /* Create the outGoing Array  */
   plhs[0] = mxCreateNumericMatrix(4, 1, mxDOUBLE_CLASS, mxREAL);
   double *labelOutPtr = mxGetPr(plhs[0]);

   labelOutPtr[0] = (double) (bestBB.ul_x + 1);
   labelOutPtr[1] = (double) (bestBB.ul_y + 1);
   labelOutPtr[2] = (double) (bestBB.lr_x + 1);
   labelOutPtr[3] = (double) (bestBB.lr_y + 1);

   //If require the energy
   if (nlhs >= 2){
      double ***integralIms = cmpIntegralIms3(ImAssign, nKeyPnt, rows, cols, weights, histSize);
      double *hist1 = bestBB.getPosNegScore(integralIms);
      double energy = hist1[0] - hist1[1];
      plhs[1] = mxCreateDoubleScalar(energy);
      destroyArrayThree<double>(integralIms, rows, cols, 2);
      destroyVector<double>(hist1, 2);
   }

   destroyMatrix<int>(ImAssign, nKeyPnt, 3);
   destroyVector<double>(weights, histSize);
}
