#include "mex.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "m_utils.h"
#include "m_malloc.h"
#include "m_TreeNode1g.h"


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){   
   /* check for the proper no. of input and outputs */
   if (nrhs != 6)
      mexErrMsgTxt("6 input arguments are required");
   if (nlhs > 3)
      mexErrMsgTxt("Too many outputs");

   int nKeyPnt = mxGetM(prhs[0]);
   int rows = (int) *mxGetPr(prhs[1]);
   int cols = (int) *mxGetPr(prhs[2]);   
   int histSize = mxGetM(prhs[3]);
   
   int Output = 0;

   /* Get the pointers to the input Data */  
   double *ImAssignPtr = mxGetPr(prhs[0]);
   double *weightsPtr = mxGetPr(prhs[3]);    
   
   /* create matrices */
   int **ImAssign;
   double *weights;  

   weights = buildVector<double>(histSize);
   ImAssign = buildMatrix<int>(nKeyPnt, 3); 
   
   // Assign the data targetHist
   for (int k=0; k<histSize; k++){
      weights[k] = (double) (*weightsPtr);
      weightsPtr++;
   }

   // Assign the data for: Im1Assign Im1Init Label1
   for (int j=0; j<3; j++){
      for (int i=0; i< nKeyPnt; i++){
         ImAssign[i][j] = ((int) (*ImAssignPtr));
         ImAssignPtr++;  
      }
   }

   double *gtBoxPtr = mxGetPr(prhs[4]);
   int *gtBox = buildVector<int>(4);
   for (int k=0; k < 4; k++){
      gtBox[k] = (int) (*gtBoxPtr);
      gtBoxPtr++;
   }
   Window gtWin(gtBox[0]-1, gtBox[1]-1, gtBox[2]-1, gtBox[3]-1);


   double *szRngPtr = mxGetPr(prhs[5]);
   int *szRng = buildVector<int>(4);

   for (int k=0; k < 4; k++){
      szRng[k] = (int) (*szRngPtr);
      szRngPtr++;
   }
   SizeRange szRange(szRng[2], szRng[3], szRng[0], szRng[1]);
   if (Output){
      printf("rows: %d, cols: %d, histSize: %d \n", rows, cols, histSize);
      printf("hmin: %d, hmax: %d, wmin: %d, wmax: %d\n", 
         szRange.hmin, szRange.hmax, szRange.wmin, szRange.wmax);
      printf("GT Box: l: %d, t: %d, r: %d, b: %d\n", 
         gtWin.ul_x, gtWin.ul_y, gtWin.lr_x, gtWin.lr_y); 
      printf("Weights[0:1]: %.20f, %.20f\n", weights[0], weights[1]);
   }  
   
   Window bestBB = findBox1g(ImAssign, nKeyPnt, rows, cols, weights, histSize, gtWin, szRange);
   /* Create the outGoing Array  */
   plhs[0] = mxCreateNumericMatrix(4, 1, mxDOUBLE_CLASS, mxREAL);   
   double *labelOutPtr = mxGetPr(plhs[0]); 
   
   labelOutPtr[0] = (double) (bestBB.ul_x + 1);
   labelOutPtr[1] = (double) (bestBB.ul_y + 1);
   labelOutPtr[2] = (double) (bestBB.lr_x + 1);
   labelOutPtr[3] = (double) (bestBB.lr_y + 1);
   
   if (nlhs > 1){
       double ***integralIms = cmpIntegralIms3(ImAssign, nKeyPnt, rows, cols, weights, histSize);
       double *hist1 = bestBB.getPosNegScore(integralIms);
       double energy = hist1[0] - hist1[1];


       Window inter1 = bestBB.intersectWin(gtWin);
       double minArea = (double)inter1.getArea();
       Window union1 = bestBB.unionWin(gtWin);
       double maxArea = (double)union1.getArea();

       double hitRateE = 0;
       if (maxArea > 0) hitRateE = minArea/maxArea;

       plhs[1] = mxCreateNumericMatrix(1, 1, mxDOUBLE_CLASS, mxREAL);
       double *energyPtr = mxGetPr(plhs[1]);
       energyPtr[0] = energy;

       plhs[2] = mxCreateNumericMatrix(1, 1, mxDOUBLE_CLASS, mxREAL);
       double *hitRatePtr = mxGetPr(plhs[2]);
       hitRatePtr[0] = hitRateE;
       if (Output){
          printf("E1: %.20f, E2: %.20f\n", energy, hitRateE);
       }  
       destroyVector<double>(hist1, 2);
       destroyArrayThree<double>(integralIms, rows, cols, 2);
   }


   destroyMatrix<int>(ImAssign, nKeyPnt, 3);
   destroyVector<double>(weights, histSize);   
   destroyVector<int>(szRng, 4);
   destroyVector<int>(gtBox, 4);
}
