#include "mex.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "m_utils.h"
#include "m_malloc.h"
#include "m_TreeNode1j.h"


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){
   /* check for the proper no. of input and outputs */
   if (nrhs != 8)
      mexErrMsgTxt("8 input arguments are required");
   if (nlhs > 2)
      mexErrMsgTxt("Too many outputs");


   int KPASSIGN_IN   = 0;
   int ROWS_IN       = 1;
   int COLS_IN       = 2;
   int B_IN          = 3;
   int SZRNG_IN      = 4;
   int ALPHA_IN      = 5;
   int TOLFAC_IN     = 6;
   int OPT_IN        = 7;

   int nKeyPnt = mxGetM(prhs[KPASSIGN_IN]);
   int rows = (int) *mxGetPr(prhs[ROWS_IN]);
   int cols = (int) *mxGetPr(prhs[COLS_IN]);
   int histSize = mxGetM(prhs[B_IN]);
   int nPC = mxGetN(prhs[B_IN]);

   bool Output = false;

   /* Get the pointers to the input Data */
   double *ImAssignPtr = mxGetPr(prhs[KPASSIGN_IN]);
   double *BPtr = mxGetPr(prhs[B_IN]);

   /* create matrices */
   int **ImAssign;
   double **B;

   ImAssign = buildMatrix<int>(nKeyPnt, 3);
   for (int j=0; j<3; j++){
      for (int i=0; i< nKeyPnt; i++){
         ImAssign[i][j] = ((int) (*ImAssignPtr));
         ImAssignPtr++;
      }
   }

   B = buildMatrix<double>(nPC, histSize);
   for (int i=0; i < nPC; i++){
      for (int j=0; j < histSize; j++){
         B[i][j] = (double) (*BPtr);
         BPtr++;
      }
   }

   double *szRngPtr = mxGetPr(prhs[SZRNG_IN]);
   int *szRng = buildVector<int>(4);
   for (int k=0; k < 4; k++){
      szRng[k] = (int) (*szRngPtr);
      szRngPtr++;
   }
   SizeRange szRange(szRng[2], szRng[3], szRng[0], szRng[1]);


   double alpha  = mxGetScalar(prhs[ALPHA_IN]);
   double tolFac = mxGetScalar(prhs[TOLFAC_IN]);
   int    option = (int) mxGetScalar(prhs[OPT_IN]);


   if (Output){
      mexPrintf("rows: %d, cols: %d, histSize: %d, nKeyPnt: %d, nPC: %d, alpha: %g, tolFact: %g, option: %d \n",
              rows, cols, histSize, nKeyPnt, nPC, alpha, tolFac, option);
      mexPrintf("hmin: %d, hmax: %d, wmin: %d, wmax: %d\n",
                  szRange.hmin, szRange.hmax, szRange.wmin, szRange.wmax);
   }

   Window bestBB;
   if (option == 1)
      bestBB = findBox1j1(ImAssign, nKeyPnt, rows, cols, B, histSize, nPC, szRange, alpha, tolFac);
   else
      bestBB = findBox1j2(ImAssign, nKeyPnt, rows, cols, B, histSize, nPC, szRange, alpha, tolFac);


   /* Create the outGoing Array  */
   plhs[0] = mxCreateNumericMatrix(4, 1, mxDOUBLE_CLASS, mxREAL);
   double *labelOutPtr = mxGetPr(plhs[0]);

   labelOutPtr[0] = (double) (bestBB.ul_x + 1);
   labelOutPtr[1] = (double) (bestBB.ul_y + 1);
   labelOutPtr[2] = (double) (bestBB.lr_x + 1);
   labelOutPtr[3] = (double) (bestBB.lr_y + 1);

   //If require the energy
   if (nlhs >= 2){
      int ***integralIms = cmpIntegralIms1(ImAssign, nKeyPnt, rows, cols, histSize);
      int *hist = bestBB.getHist(integralIms, histSize);
      double energy;
      if (option == 1)
         energy = cmpEnergy3(hist, B, histSize, nPC, alpha);
      else
         energy = cmpEnergy4(hist, B, histSize, nPC, alpha);
      plhs[1] = mxCreateDoubleScalar(energy);
      destroyArrayThree<int>(integralIms, rows, cols, histSize);
      destroyVector<int>(hist, histSize);
   }

   destroyMatrix<int>(ImAssign, nKeyPnt, 3);
   destroyMatrix<double>(B, nPC, histSize);
   destroyVector<int>(szRng, 4);
}
