#include "mex.h"
#include <cmath>
#include <stdlib.h>
#include <stdio.h>
#include <queue>
#include "m_utils.h"
#include "m_malloc.h"
#include "m_Multi1DSegs.h"

using namespace std;

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){   
   /* check for the proper no. of input and outputs */
   if (nrhs != 4)
      mexErrMsgTxt("4 input arguments are required");
   if (nlhs>1)
      mexErrMsgTxt("Too many outputs");

   int nKeyPnt = mxGetM(prhs[0]);
   int cols = (int) *mxGetPr(prhs[1]);
   
   int histSize = mxGetM(prhs[2]);

   /* Get the pointers to the input Data */  
   double *ImAssignPtr = mxGetPr(prhs[0]);
   double *weightsPtr = mxGetPr(prhs[2]);    
   double *nSegPtr = mxGetPr(prhs[3]);

   int nSeg = (int) *nSegPtr; 
   
   /* create matrices */
   int **ImAssign;
   double *weights;  

   weights = buildVector<double>(histSize);
   ImAssign = buildMatrix<int>(nKeyPnt, 3); 
   
   // Assign the data targetHist
   for (int k=0; k<histSize; k++){
      weights[k] = (double) (*weightsPtr);
      weightsPtr++;
   }

   // Assign the data for ImAssign
   for (int i=0; i< nKeyPnt; i++){
      ImAssign[i][0] = 1;
   }
   for (int j=1; j<3; j++){
      for (int i=0; i< nKeyPnt; i++){
         ImAssign[i][j] = ((int) (*ImAssignPtr));
         ImAssignPtr++;  
      }
   }


   
   bool Output = 0;
   if (Output){
      printf("cols: %d, histSize: %d, nKeyPnt: %d, nSeg: %d\n", 
              cols, histSize, nKeyPnt, nSeg);      
   }   
   
   int nChosen;
   double **chosenSegs = findMulti1DSegs(ImAssign, nKeyPnt, cols, weights, 
      histSize, nSeg, &nChosen); 

   if (Output){
      printf("# chosen segments: %d\n", nChosen);  
   }
 
   
   /*if (!isImAssignGood(ImAssign, rows, cols)){
      mexErrMsgTxt("ImAssign, 1st agrument must be matrix of positive integers");
   }*/      

   /* Create the outGoing Array  */
   plhs[0] = mxCreateNumericMatrix(3, nChosen, mxDOUBLE_CLASS, mxREAL);   
   double *labelOutPtr = mxGetPr(plhs[0]); 

   for (int k=0; k < nChosen; k++){    
      labelOutPtr[0] = (double) (chosenSegs[0][k] + 1);
      labelOutPtr[1] = (double) (chosenSegs[1][k] + 1);
      labelOutPtr[2] = (double) chosenSegs[2][k];
      labelOutPtr += 3;      
   }

   destroyMatrix<int>(ImAssign, nKeyPnt, 3); 
   destroyVector<double>(weights, histSize);
   destroyMatrix<double>(chosenSegs, 3, nChosen);
}
