#include "mex.h"
#include <cmath>
#include <stdlib.h>
#include <stdio.h>
#include <queue>
#include "m_utils.h"
#include "m_malloc.h"
#include "m_Multi1DSegs.h"

using namespace std;

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){
   /* check for the proper no. of input and outputs */
   if (nrhs != 4) mexErrMsgTxt("4 input arguments are required");
   if (nlhs>1) mexErrMsgTxt("Too many outputs");

   int PIXVALS_IN = 0;
   int NSEG_IN    = 1;
   int SZRNG_IN   = 2;
   int THRES_IN   = 3;

   int len = mxGetM(prhs[PIXVALS_IN]);
   double *pixValsPtr = mxGetPr(prhs[PIXVALS_IN]);
   double *pixVals = (double *) mxCalloc(len, sizeof(double));
   for (int i=0; i < len; i++) pixVals[i] = pixValsPtr[i];
   int nSeg = (int) mxGetScalar(prhs[NSEG_IN]);

   double thres = mxGetScalar(prhs[THRES_IN]);

   double *szRngPtr = mxGetPr(prhs[SZRNG_IN]);
   int *szRng = (int*) mxCalloc(2, sizeof(int));
   for (int k=0; k < 2; k++) szRng[k] = (int) szRngPtr[k];
   SizeRange szRange(1, 1, szRng[0], szRng[1]);


   int nChosen;
   double **chosenSegs = findMulti1DSegs_seq2(pixVals, len, nSeg, szRange, thres, &nChosen);


   /* Create the outGoing Array  */
   plhs[0] = mxCreateNumericMatrix(3, nChosen, mxDOUBLE_CLASS, mxREAL);
   double *labelOutPtr = mxGetPr(plhs[0]);

   for (int k=0; k < nChosen; k++){
      labelOutPtr[0] = (double) (chosenSegs[0][k] + 1);
      labelOutPtr[1] = (double) (chosenSegs[1][k] + 1);
      labelOutPtr[2] = (double) chosenSegs[2][k];
      labelOutPtr += 3;
   }

   destroyMatrix<double>(chosenSegs, 3, nChosen);
}
