/*
 * m_mexMMCSeg.cpp
 *
 * Created on: Sep 17, 2010
 * Author: Minh Hoai Nguyen, Carnegie Mellon University
 * Email:  minhhoai@cmu.edu, minhhoai@gmail.com
 */
#include "mex.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "m_utils.h"
#include "m_malloc.h"
#include "m_multiLabelKSeg.h"


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){
   /* check for the proper no. of input and outputs */
   if (nrhs != 3)
      mexErrMsgTxt("Three input arguments are required");
   if (nlhs > 2)
      mexErrMsgTxt("Too many outputs");

   int A_IN    = 0;
   int LMIN_IN = 1;
   int LMAX_IN = 2;

   int len    = mxGetM(prhs[A_IN]);
   int nLabel = mxGetN(prhs[A_IN]);
   int lmin = (int) mxGetScalar(prhs[LMIN_IN]);
   int lmax = (int) mxGetScalar(prhs[LMAX_IN]);

   bool Output = false;

   /* Get the pointers to the input Data */
   double *APtr = mxGetPr(prhs[A_IN]);

   double **A = buildMatrix<double>(nLabel, len);
   for (int j = 0; j < nLabel; j++){
      memcpy(A[j], APtr, len*sizeof(double));
      APtr += len;
   }

   if (Output) mexPrintf("nLabel: %d, len: %d, lmin: %d, lmax: %d\n",
         nLabel, len, lmin, lmax);

   double energy;
   vector<MLKS_Rslt> rslt = MMCSeg(A, nLabel, len, lmin, lmax, &energy);

   /* Create the outGoing Array  */
   plhs[0] = mxCreateDoubleMatrix(3, rslt.size(), mxREAL);
   double *assgn = mxGetPr(plhs[0]);

   for (int j = (int) rslt.size() -1; j >=0; j--){
      *assgn++ = rslt[j].startIdx + 1;
      *assgn++ = rslt[j].endIdx + 1;
      *assgn++ = rslt[j].label + 1;
   }

   plhs[1] = mxCreateDoubleScalar(energy);
   destroyMatrix<double>(A, nLabel, len);
}
