function [bestBox, E1, E2] = ml_findBox1g2(pixVals, gtWin, szRange)
% [bestBox, E1, E2] = ml_findBox1g2(pixVals, gtWin, szRange)
% Find a rectangle with the histogram that minimize the sum
%   sum_{i in win}pixVals(i) + area(intersect(win, gtWin))/area(union(win,gtWin))
% Inputs:
%   pixVals: imH*imW matrix 
%   imH, imW: height and width of the image
%   gtWin: a 4*1 vector for [left, top, right, bottom] of the gt window
%	szRange: a 4*1 vector for [wmin; wmax; hmin; hmax] range for width and height 
% Output:
%   bestBox: a 4*1 vector for the left, top, right, bottom of the rectangle
%   E1: sum_{i in win} pixVals(i)
%   E2: area(intersect(win, gtWin))/area(union(win,gtWin))
% Examples:
%   pixVals = ones(100, 200);
%   pixVals(50:70, 80:120) = -1;
%   [bestBox, e1, e2] = ml_findBox1g2(pixVals, [80;50;120;70]); 
%       ==> should get [80; 50; 120; 70];
% Note: This is to find most violated constraint for structure output learning
% See also: 
%   ml_findBox1b, ml_findBox1b2
%   ml_findBox1g: serves similar purpose but with dif input format.
% By: Minh Hoai Nguyen (minhhoai@gmail.com)
% Date: 6 Aug 09

[imH, imW] = size(pixVals);

if exist('szRange','var') && ~isempty(szRange)
    if (size(szRange, 1) ~= 4) || (size(szRange, 2) ~= 1)
        error('szRange should be a 4*1 vector');
    end;

    if (szRange(1) > szRange(2)) || (szRange(3) > szRange(4)) ||...
        szRange(2) <= 0 || szRange(4) <=0 || ...
        szRange(1) > imW || szRange(3) > imH
        error('invalid size range');
    end;
else 
    szRange = [0; imW; 0; imH];
end;

if ~exist('gtWin', 'var') || isempty(gtWin)
    gtWin = [-1; -1; -1; -1];
end;

if nargout == 1
    bestBox = m_mexFindBox1g2(pixVals, gtWin, szRange);
elseif nargout == 2
    [bestBox, E1] = m_mexFindBox1g2(pixVals, gtWin, szRange);
else
    [bestBox, E1, E2] = m_mexFindBox1g2(pixVals, gtWin, szRange);
end
    
    
    




