function Segs = ml_findMulti1DSegs2(pixVals, nSeg)
% function Segs = ml_findMulti1DSegs2(pixVals, nSeg)
% Finding multiple 1D segments from a signal such that the sum of
% elements in those segments is minimized.
% This function find GLOBAL minimum!
% Inputs:
%   pixVals: a imH*1 vector
%   nSeg: number of segments to extract, must a pos integer. The default is 1
% Output:
%   Segs: a 3*K matrix with K <= nSeg. Segs([1,2],i) marks the left and
%   right ends of the interval. Segs(3,i) is the value of the interval.
% Examples:
%   pixVals = ones(40,1);
%   pixVals([4:7, 10:15, 25:35]) = -1;
%   ml_findMulti1DSegs2(pixVals, 3)
%   ml_findMulti1DSegs2(pixVals, 2)
%   ml_findMulti1DSegs2(pixVals, 1)
%   
% See also:
%   ml_findMulti1DSegs.m serves the same purpose but uses dif input format
%   ml_findMulti1DSegs_seq.m, ml_findMulti1DSegs_seq2.m do not guarantee to 
%   find global minimum. However, it can incorporate size constraint.
%   
% By: Minh Hoai Nguyen (minhhoai@gmail.com)
% Date: 6 Aug 2009

if size(pixVals, 2) ~= 1
    error('pixVals must be a column vector');
end

if (nSeg < 1)
    error('Number of segment requested must be at least 1');
end;

Segs = m_mexFindMulti1DSegs2(pixVals, nSeg);



