#include <iostream>
#include <cmath>
#include <queue>
#include <string>
#include <sstream>
#include <fstream>
#include "m_utils.h"
#include "m_malloc.h"
#include "m_Multi1DSegs.h"
#include "m_BranchBound.cpp"
#include "m_tests.h"


class Multi1DSegsTest{
public:
   int **imAssign, imW, nHistBin, nSeg, nChosenSeg;
   double *weights, expVal;
   int **expSegs;


   Multi1DSegsTest(int imW_, int nHistBin_, int nSeg_, double *weights_){
      nHistBin = nHistBin_;
      imW = imW_;
      nSeg = nSeg_;
      weights = buildVector<double>(nHistBin);
      for (int i=0;i < nHistBin; i++){
         weights[i] = weights_[i];
      }
      imAssign = buildMatrix<int>(1, imW);
   }


   void setVal(int l, int u, int binNo){ //assign histogram bin number
      for (int i=l; i <= u; i++){
         imAssign[0][i] = binNo;
      }
   };

   void setExpSegs(int *expSegs_, int nChosenSeg_){
      nChosenSeg = nChosenSeg_;
      expSegs = buildMatrix<int>(2, nChosenSeg);
      expVal = 0;
      for (int i=0; i < nChosenSeg; i++){
         expSegs[0][i] = expSegs_[2*i];
         expSegs[1][i] = expSegs_[2*i+1];

         double ***integralIms = cmpIntegralIms2(imAssign, 1, imW, weights, nHistBin);
         Window win(expSegs[0][i], 0, expSegs[1][i], 0);
         double *hist = win.getPosNegScore(integralIms);
         expVal += hist[0] - hist[1];
         destroyArrayThree<double>(integralIms, 1, imW, 2);
         destroyVector<double>(hist, 2);
      }
   };

   ~Multi1DSegsTest(){
      destroyMatrix<int>(imAssign, 1, imW);
      destroyVector<double>(weights, nHistBin);
      if (nChosenSeg > 0) destroyMatrix<int>(expSegs, 2, nChosenSeg);
   };
};


class Multi1DSegsTest2:public Multi1DSegsTest{
public:
   SizeRange szRange;
   double thres;

   Multi1DSegsTest2(int imW_, int nHistBin_, int nSeg_, double *weights_,
      SizeRange szRange_, double thres_):Multi1DSegsTest(imW_, nHistBin_, nSeg_, weights_){
         szRange = szRange_;
         thres = thres_;
   }
};

void testMulti1DSegs(){
   double weights[] = {3, -1};


   queue<Multi1DSegsTest*> tests;
   int imW = 20;
   int nSeg = 2;
   int nHistBin = 2;
   Multi1DSegsTest test1(imW, nHistBin, nSeg, weights);
   test1.setVal(0, imW-1, 1);
   test1.expVal = 0;
   test1.nChosenSeg = 0;


   Multi1DSegsTest test2(imW, nHistBin, nSeg, weights);
   test2.setVal(0, imW-1, 1);
   test2.setVal(3, 7, 2);
   int expSegs2[] = {3, 7};
   test2.setExpSegs(expSegs2, 1);

   Multi1DSegsTest test3(imW, nHistBin, nSeg, weights);
   test3.setVal(0, imW-1, 1);
   test3.setVal(3, 7, 2);
   test3.setVal(10, 15, 2);
   int expSegs3[] = {3, 7, 10, 15};
   test3.setExpSegs(expSegs3, 2);


   imW = 40;
   Multi1DSegsTest test4(imW, nHistBin, nSeg, weights);
   test4.setVal(0, imW-1, 1);
   test4.setVal(3, 7, 2);
   test4.setVal(10, 15, 2);
   test4.setVal(25, 35, 2);
   int expSegs4[] = {10, 15, 25, 35};
   test4.setExpSegs(expSegs4, 2);

   double weights5[] = {1, -1};
   Multi1DSegsTest test5(imW, nHistBin, nSeg, weights5);
   test5.setVal(0, imW-1, 1);
   test5.setVal(3, 7, 2);
   test5.setVal(10, 15, 2);
   test5.setVal(25, 35, 2);
   int expSegs5[] = {3, 15, 25, 35};
   test5.setExpSegs(expSegs5, 2);

   Multi1DSegsTest test6(imW, nHistBin, 3, weights5);
   test6.setVal(0, imW-1, 1);
   test6.setVal(3, 7, 2);
   test6.setVal(10, 15, 2);
   test6.setVal(25, 35, 2);
   int expSegs6[] = {3, 7, 10, 15, 25, 35};
   test6.setExpSegs(expSegs6, 3);


   Multi1DSegsTest test7(imW, nHistBin, 7, weights5);
   test7.setVal(0, imW-1, 1);
   test7.setVal(3, 7, 2);
   test7.setVal(10, 15, 2);
   test7.setVal(25, 35, 2);
   int expSegs7[] = {3, 7, 10, 15, 25, 35};
   test7.setExpSegs(expSegs7, 3);

   double weights8[] = {2, -1};
   Multi1DSegsTest test8(imW, nHistBin, 2, weights8);
   test8.setVal(0, imW-1, 1);
   test8.setVal(3, 6, 2);
   test8.setVal(10, 20, 2);
   test8.setVal(23, 35, 2);
   int expSegs8[] = {10, 20, 23, 35};
   test8.setExpSegs(expSegs8, 2);

   Multi1DSegsTest test9(imW, nHistBin, 1, weights8);
   test9.setVal(0, imW-1, 1);
   test9.setVal(3, 6, 2);
   test9.setVal(10, 20, 2);
   test9.setVal(25, 35, 2);
   int expSegs9[] = {10, 35};
   test9.setExpSegs(expSegs9, 1);

   Multi1DSegsTest test10(imW, nHistBin, 3, weights8);
   test10.setVal(0, imW-1, 2);
   int expSegs10[] = {0, imW-1};
   test10.setExpSegs(expSegs10, 1);

   tests.push(&test1);
   tests.push(&test2);
   tests.push(&test3);
   tests.push(&test4);
   tests.push(&test5);
   tests.push(&test6);
   tests.push(&test7);
   tests.push(&test8);
   tests.push(&test9);
   tests.push(&test10);

   int i= 0;
   while (!tests.empty()){
      Multi1DSegsTest *test = tests.front();
      tests.pop();
      i++; cout << "==============Testing Multi1DSegs " << i << endl;
      for (int k=0; k < test->imW; k++){
         cout << " " << test->imAssign[0][k];
      }
      cout << endl;


      int** kpAssign = imAssign2kpAssign(test->imAssign, 1, test->imW);
      int nChosen;
      double **chosenSegs = findMulti1DSegs(kpAssign, test->imW, test->imW,
         test->weights, test->nHistBin, test->nSeg, &nChosen);

      destroyMatrix<int>(kpAssign, test->imW, 3);

      double bestRetVal = 0; // best return value.
      cout << "  # selected ints: " << nChosen << "==> ";

      bestRetVal = 0;
      for (int k=0; k < nChosen; k++){
         bestRetVal += chosenSegs[2][k];
         cout << "[" << chosenSegs[0][k] << ", "
              << chosenSegs[1][k] << "]; " ;
      }
      cout << endl;

      if (bestRetVal < test->expVal){
         cout << "WARNING: expected energy is bigger than returned energy" << endl;
         cout << bestRetVal << " " << test->expVal << endl;
      } else if (bestRetVal > test->expVal){
         cout << "FAILED, expected E: " << test->expVal << ", return: " << bestRetVal << endl;
         cout << "  expected: ";
         for (int k = 0; k < test->nChosenSeg; k++){
            cout << "[" << test->expSegs[0][k] << ", "
               << test->expSegs[1][k] << "]; ";
         }
         cout << endl;

      } else {
         cout << "PASSED" << endl;
      }
      destroyMatrix<double>(chosenSegs, 3, nChosen);
   }
}

void testMulti1DSegs_2(){
   double weights[] = {3, -1};

   queue<Multi1DSegsTest2*> tests;
   int imW = 20;
   int nSeg = 2;
   int nHistBin = 2;
   SizeRange szRange(1, 1, 5, 10);
   double thres = 0;
   Multi1DSegsTest2 test1(imW, nHistBin, nSeg, weights, szRange, thres);
   test1.setVal(0, imW-1, 1);
   test1.expVal = 0;
   test1.nChosenSeg = 0;


   SizeRange szRange2(1,1,3, 10);
   Multi1DSegsTest2 test2(imW, nHistBin, nSeg, weights, szRange2, thres);
   test2.setVal(0, imW-1, 1);
   test2.setVal(3, 7, 2);
   int expSegs2[] = {3, 7};
   test2.setExpSegs(expSegs2, 1);

   Multi1DSegsTest2 test3(imW, nHistBin, nSeg, weights, szRange, thres);
   test3.setVal(0, imW-1, 1);
   test3.setVal(3, 7, 2);
   test3.setVal(10, 15, 2);
   int expSegs3[] = {3, 7, 10, 15};
   test3.setExpSegs(expSegs3, 2);


   imW = 40;
   Multi1DSegsTest2 test4(imW, nHistBin, nSeg, weights, szRange, thres);
   test4.setVal(0, imW-1, 1);
   test4.setVal(3, 7, 2);
   test4.setVal(10, 15, 2);
   test4.setVal(24, 35, 2);
   int expSegs4[] = {10, 15, 26, 35};
   test4.setExpSegs(expSegs4, 2);

   double weights5[] = {1, -2};
   Multi1DSegsTest2 test5(imW, nHistBin, nSeg, weights5, szRange, thres);
   test5.setVal(0, imW-1, 1);
   test5.setVal(3, 7, 2);
   test5.setVal(10, 15, 2);
   test5.setVal(25, 35, 2);
   int expSegs5[] = {6, 15, 26, 35};
   test5.setExpSegs(expSegs5, 2);

   Multi1DSegsTest2 test6(imW, nHistBin, 3, weights5, szRange, thres);
   test6.setVal(0, imW-1, 1);
   test6.setVal(4, 6, 2);
   test6.setVal(10, 15, 2);
   test6.setVal(25, 35, 2);
   int expSegs6[] = {2, 6, 10, 15, 26, 35};
   test6.setExpSegs(expSegs6, 3);


   Multi1DSegsTest2 test7(imW, nHistBin, 7, weights, szRange, thres);
   test7.setVal(0, imW-1, 1);
   test7.setVal(3, 4, 2);
   test7.setVal(10, 15, 2);
   test7.setVal(25, 35, 2);
   int expSegs7[] = {10, 15, 25, 34};
   test7.setExpSegs(expSegs7, 2);

   double weights8[] = {10, -1};
   Multi1DSegsTest2 test8(imW, nHistBin, 3, weights8, szRange, thres);
   test8.setVal(0, imW-1, 1);
   test8.setVal(3, 6, 2);
   test8.setVal(10, 20, 2);
   test8.setVal(23, 35, 2);
   int expSegs8[] = {11, 20, 26, 35};
   test8.setExpSegs(expSegs8, 2);

   thres = -7;
   Multi1DSegsTest2 test9(imW, nHistBin, 3, weights8, szRange, thres);
   test9.setVal(0, imW-1, 1);
   test9.setVal(3, 6, 2);
   test9.setVal(10, 15, 2);
   test9.setVal(25, 35, 2);
   int expSegs9[] = {26, 35};
   test9.setExpSegs(expSegs9, 1);

   Multi1DSegsTest2 test10(imW, nHistBin, 3, weights8, szRange, thres);
   test10.setVal(0, imW-1, 2);
   int expSegs10[] = {0, 9, 10, 19, 20, 29};
   test10.setExpSegs(expSegs10, 3);

   thres = -7;
   Multi1DSegsTest2 test11(imW, nHistBin, -1, weights, szRange, thres);
   test11.setVal(0, imW-1, 1);
   test11.setVal(3, 6, 2);
   test11.setVal(10, 17, 2);
   test11.setVal(25, 35, 2);
   int expSegs11[] = {10, 17, 26, 35};
   test11.setExpSegs(expSegs11, 2);

   thres = -2;
   Multi1DSegsTest2 test12(imW, nHistBin, -1, weights, szRange, thres);
   test12.setVal(0, imW-1, 1);
   test12.setVal(3, 6, 2);
   test12.setVal(10, 17, 2);
   test12.setVal(25, 35, 2);
   int expSegs12[] = {10, 17, 26, 35};
   test12.setExpSegs(expSegs12, 2);

   thres = -2;
   Multi1DSegsTest2 test13(imW, nHistBin, -1, weights5, szRange, thres);
   test13.setVal(0, imW-1, 1);
   test13.setVal(3, 6, 2);
   test13.setVal(10, 17, 2);
   test13.setVal(25, 35, 2);
   int expSegs13[] = {2, 6, 10, 17, 26, 35};
   test13.setExpSegs(expSegs13, 3);

   tests.push(&test1);
   tests.push(&test2);
   tests.push(&test3);
   tests.push(&test4);
   tests.push(&test5);
   tests.push(&test6);
   tests.push(&test7);
   tests.push(&test8);
   tests.push(&test9);
   tests.push(&test10);
   tests.push(&test11);
   tests.push(&test12);
   tests.push(&test13);

   int i= 0;
   while (!tests.empty()){
      Multi1DSegsTest2 *test = tests.front();
      tests.pop();
      i++; cout << "==============Testing Multi1DSegs_2 " << i << endl;
      for (int k=0; k < test->imW; k++){
         cout << " " << test->imAssign[0][k];
      }
      cout << endl;

      int** kpAssign = imAssign2kpAssign(test->imAssign, 1, test->imW);

      int nChosen;
      double **chosenSegs = findMulti1DSegs_seq(kpAssign, test->imW, test->imW,
         test->weights, test->nHistBin, test->nSeg, test->szRange, test->thres,
         &nChosen);
      destroyMatrix<int>(kpAssign, test->imW, 3);

      double bestRetVal = 0; // best return value.
      cout << "  # selected ints: " << nChosen << "==> ";

      bestRetVal = 0;
      for (int k=0; k < nChosen; k++) {
         bestRetVal += chosenSegs[2][k];
         cout << "[" << chosenSegs[0][k] << ", "
              << chosenSegs[1][k] << "]; " ;
      }
      cout << endl;

      if (bestRetVal < test->expVal){
         cout << "WARNING: expected energy is bigger than returned energy" << endl;
         cout << bestRetVal << " " << test->expVal << endl;
      } else if (bestRetVal > test->expVal){
         cout << "FAILED, expected E: " << test->expVal << ", return: " << bestRetVal << endl;
         cout << "  expected: ";
         for (int k = 0; k < test->nChosenSeg; k++){
            cout << "[" << test->expSegs[0][k] << ", "
               << test->expSegs[1][k] << "]; ";
         }
         cout << endl;

      } else {
         cout << "PASSED" << endl;
      }
      destroyMatrix<double>(chosenSegs, 3, nChosen);
   }
}
