#ifndef M_TREENODE1F_H
#define M_TREENODE1F_H
#include <vector>
#include "m_TreeNode1e.h"

class IntImTree{
public:
   int imH, imW, nBranch, sqrtBranch, nLevel;
   double ***integralIms;
   bool isLeaf;
   vector<IntImTree* > branches;

   // Constructors

   /**
    * Create a tree of integral images
    * Inputs:
    *    weightss_: a 2D array for several pow(nBranch_, nLevel_) weight vectors.
    *       the tree has nBranch_, the first pow(nBranch_, nLevel_ -1) weight
    *       vectors are used to built the first branch and so on so forth.
    *    nBranch_: number of branches
    *    nLevel_: number of levels
    */
   IntImTree(int **kpAssign, int nKeyPnt, int imH, int imW,
             double **weightss_, int nHistBin_, int nBranch_, int nLevel_);
   IntImTree(){};
   ~IntImTree();


   //functions
   double cmpLb(WindowPair winPair);
   double cmpE(Window win);

};

class TreeNode1f:public TreeNode1e{
public:
   static IntImTree *intImTree;
   static SizeRange szRange;
   int sqrtBrnch;

   void cmpBnds();

   TreeNode1f(WindowPair winPair_):TreeNode1e(winPair_){};
   TreeNode1f():TreeNode1e(){};
};

/**
 * Find the best box in an image minimize a pyramid of weights.
 *    like in the pyramid matching kernel.
 *    by minimizing the objective:
 *    sum_ij weights(i,j)*hist_i(j)
 * Inputs:
 *    kpAssign: a 2-dim array of size nKeyPnt*3 for histogram bin assignments.
 *    kpAssign[i][0], kpAssign[i][1], kpAssign[i][2] are row#, col#, bin#
 *       entries are positive integers.
 *    nKeyPnt: # of keypoints.
 *    imH, imW: height and width of the image
 *    weightTree: a tree for weights at all levels, all cells of the pyramid.
 *    nHistBin: # of histogram bins
 *    szRange: range for the width and height of the box.
 *    tolFactor: tolerance factor for stopping criteria of branch and bound
 *       if tolFactor = 0, run BB until global optimality is known.
 *       the bigger tolFactor, the earlier BB terminates.
 *    energy: the energy of the returned output.
 * Output:
 *    A best (within tolerance) rectangle that minimize the pyramid of weights.
 */
Window findBox1f(int **kpAssign, int nKeyPnt, int imH, int imW, double **weightss,
                 int nHistBin, int nBranch, int nLevel, SizeRange szRange,
                 double tolFac, double *energy);


#endif
