#ifndef M_TREENODE1G_H
#define M_TREENODE1G_H
#include "m_TreeNode1e.h"

class TreeNode1g:public TreeNode1e{
public:
   static double ***integralIms;
   static SizeRange szRange;

   // ground truth window, this is necessary to generate
   // window with maximum loss + score
   static Window gtWin;

   TreeNode1g(WindowPair winPair_):TreeNode1e(winPair_){};
   TreeNode1g():TreeNode1e(){};
   void cmpBnds();


};

/**
 * Find the best box in an image that minimizes:
 *    sum_i {weights(i)*hist(i)} +
 *       Area(intersect(returnWin,gtWin))/Area(union(returnWin,gtWin)).
 * Inputs:
 *    kpAssign: a 2-dim array of size nKeyPnt*3 for histogram bin assignments.
 *    kpAssign[i][0], kpAssign[i][1], kpAssign[i][2] are row#, col#, bin#
 *       entries are positive integers.
 *    nKeyPnt: # of keypoints.
 *    imH, imW: height and width of the image
 *    weights: a 1-dim array for weights for histogram bins
 *    nHistBin: # of histogram bins
 *    gtWin: ground truth window
 *    szRange: range for the width and height of the box.
 * Output:
 *    A best rectangle that minimizes the desired energy
 */
Window findBox1g(int **kpAssign, int nKeyPnt, int imH, int imW,
                 double *weights, int nHistBin, Window gtWin,
                 SizeRange szRange);


/**
 * Find the best box in an image that minimizes:
 *    sum_{i in returnWin} pixVals(i) +
 *       Area(intersect(returnWin,gtWin))/Area(union(returnWin,gtWin)).
 * This function is to find the most violated constraint
 * Inputs:
 *    pixVals: imH*imW image of values
 *    imH, imW: height and width of the image
 *    gtWin: gt window
 *    szRange: range for the width and height of the box.
 * Output:
 *    A best rectangle that minimizes the desired energy
 */
Window findBox1g2(double **pixVals, int imH, int imW, Window gtWin, SizeRange szRange);



#endif
