#include <iostream>
#include <fstream>
#include <cmath>
#include <queue>
#include <string>
#include <sstream>
#include "m_malloc.h"
#include "m_utils.h"
#include "time.h"
#include "m_TreeNode2a.h"
#include "m_TreeNode1a.h"
#include "m_BranchBound.cpp"

int ***TreeNode2a::integralIms1 = NULL;
int ***TreeNode2a::integralIms2 = NULL;
int TreeNode2a::nHistBin = 0;
double *TreeNode2a::weights = NULL;
double TreeNode2a::C = 0;


void
TreeNode2a::cmpBnds(){
   Window smallestWin1 = winPair1.getSmallestWin();
   int *hist1_min = smallestWin1.getHist(integralIms1, nHistBin);
   Window largestWin1 = winPair1.getLargestWin();
   int *hist1_max = largestWin1.getHist(integralIms1, nHistBin);

   Window smallestWin2 = winPair2.getSmallestWin();
   int *hist2_min = smallestWin2.getHist(integralIms2, nHistBin);
   Window largestWin2 = winPair2.getLargestWin();
   int *hist2_max = largestWin2.getHist(integralIms2, nHistBin);

   lb = getLowerBnd2(hist1_min, hist1_max, hist2_min, hist2_max, weights, nHistBin, C);

   double ub1, ub2, ub3, ub4; // four candidates
   ub1 = cmpEnergy(hist1_min, hist2_min, weights, nHistBin, C);

   ub = ub1;
   bestBBoxes = WindowPair(smallestWin1, smallestWin2);

   ub2 = cmpEnergy(hist1_min, hist2_max, weights, nHistBin, C);

   if (ub > ub2){
      ub = ub2;
      bestBBoxes = WindowPair(smallestWin1, largestWin2);
   }

   ub3 = cmpEnergy(hist1_max, hist2_min, weights, nHistBin, C);

   if (ub > ub3){
      ub = ub3;
      bestBBoxes = WindowPair(largestWin1, smallestWin2);
   }

   ub4 = cmpEnergy(hist1_max, hist2_max, weights, nHistBin, C);

   if (ub > ub4){
      ub = ub4;
      bestBBoxes = WindowPair(largestWin1, largestWin2);
   }

   //ub = min(min(ub1, ub2), min(ub3, ub4));

   destroyVector<int>(hist1_min, nHistBin);
   destroyVector<int>(hist1_max, nHistBin);
   destroyVector<int>(hist2_min, nHistBin);
   destroyVector<int>(hist2_max, nHistBin);

   cmpUb1();
};

void
TreeNode2a::cmpUb1(){
   Window randWin2 = winPair2.getRandomWin();
   Window randWin1 = winPair1.getRandomWin();
   int *hist2_rand = randWin2.getHist(integralIms2, nHistBin);
   int *hist1_rand = randWin1.getHist(integralIms1, nHistBin);

   double newUb = cmpEnergy(hist1_rand, hist2_rand, weights, nHistBin, C);
   if (ub > newUb){
      ub = newUb;
      bestBBoxes = WindowPair(randWin1, randWin2);
   }

   destroyVector<int>(hist1_rand, nHistBin);
   destroyVector<int>(hist2_rand, nHistBin);
}

void
TreeNode2a::cmpUb2(double knownBest){
   Window smallestWin2 = winPair2.getSmallestWin();
   int *hist2_min = smallestWin2.getHist(integralIms2, nHistBin);
   Window largestWin2 = winPair2.getLargestWin();
   int *hist2_max = largestWin2.getHist(integralIms2, nHistBin);

   Window bestWin1a = findBox1a_helper(integralIms1, TreeNode1a(winPair1),
      hist2_min, weights, nHistBin, C, 0, knownBest);
   Window bestWin1b = findBox1a_helper(integralIms1, TreeNode1a(winPair1),
      hist2_max, weights, nHistBin, C, 0, knownBest);

   int *bestWin1a_hist = bestWin1a.getHist(integralIms1, nHistBin);
   int *bestWin1b_hist = bestWin1b.getHist(integralIms1, nHistBin);

   double ub5 = cmpEnergy(bestWin1a_hist, hist2_min, weights, nHistBin, C);
   double ub6 = cmpEnergy(bestWin1b_hist, hist2_max, weights, nHistBin, C);
   ub = min(ub, min(ub5, ub6));

   destroyVector<int>(bestWin1a_hist, nHistBin);
   destroyVector<int>(bestWin1b_hist, nHistBin);
   destroyVector<int>(hist2_min, nHistBin);
   destroyVector<int>(hist2_max, nHistBin);
}


void
TreeNode2a::cmpUb3(double knownBest){
   Window randWin2 = winPair2.getRandomWin();
   int *hist2_rand = randWin2.getHist(integralIms2, nHistBin);

   Window bestWin1 = findBox1a_helper(integralIms1, TreeNode1a(winPair1),
      hist2_rand, weights, nHistBin, C, 0, knownBest);

   int *bestWin1_hist = bestWin1.getHist(integralIms1, nHistBin);

   double ub5 = cmpEnergy(bestWin1_hist, hist2_rand, weights, nHistBin, C);
   if (ub > ub5){
      ub = ub5;
      bestBBoxes = WindowPair(bestWin1, randWin2);
   }

   destroyVector<int>(bestWin1_hist, nHistBin);
   destroyVector<int>(hist2_rand, nHistBin);
}




typedef priority_queue<TreeNode2a, vector<TreeNode2a>, TreeNodeCompare<TreeNode2a> > TreeNode2Queue;

TreeNode2Queue pruneQueue(TreeNode2Queue myQ, double pruneVal){
   vector<TreeNode2a> remainElems;
   int queueSz = myQ.size();
   for (int i = 0; i < queueSz; i++){
      TreeNode2a topNode = myQ.top();
      myQ.pop();
      if (topNode.lb <= pruneVal){
         remainElems.push_back(topNode);
      }
   }
   TreeNode2Queue prunedQueue(remainElems.begin(), remainElems.end());
   return prunedQueue;

}


WindowPair findBoxes2a(int **imAssign1, int imH1, int imW1,
                       int **imAssign2, int imH2, int imW2,
                       double *weights, int nHistBin,
                       double C, double tolFactor, double knownBest){

   int ***integralIms1 = cmpIntegralIms(imAssign1, imH1, imW1, nHistBin);
   int ***integralIms2 = cmpIntegralIms(imAssign2, imH2, imW2, nHistBin);

   TreeNode2a::integralIms1 = integralIms1;
   TreeNode2a::integralIms2 = integralIms2;
   TreeNode2a::nHistBin = nHistBin;
   TreeNode2a::weights = weights;
   TreeNode2a::C = C;


   Window biggestWin1(0,0, imW1-1, imH1-1);
   Window biggestWin2(0,0, imW2-1, imH2-1);

   TreeNode2a rootNode(WindowPair(biggestWin1, biggestWin1),
                      WindowPair(biggestWin2, biggestWin2));

   TreeNode2a bestNode = m_branch_bound<TreeNode2a>(rootNode, tolFactor);

   destroyArrayThree<int>(integralIms1, imH1, imW1, nHistBin);
   destroyArrayThree<int>(integralIms2, imH2, imW2, nHistBin);
   return bestNode.bestBBoxes;
}
