/*
 * m_TreeNode2c.cpp
 *
 * Created on: Oct 25, 2009
 * Author: Minh Hoai Nguyen, Carnegie Mellon University
 * Email:  minhhoai@cmu.edu, minhhoai@gmail.com
 */
#include <iostream>
#include <fstream>
#include <cmath>
#include <queue>
#include <string>
#include <sstream>
#include "m_malloc.h"
#include "m_utils.h"
#include "m_BranchBound.cpp"
#include "m_TreeNode2c.h"


int ***TreeNode2c::integralIms1 = NULL;
int ***TreeNode2c::integralIms2 = NULL;
int TreeNode2c::nHistBin = 0;
SizeRange TreeNode2c::szRange1;
SizeRange TreeNode2c::szRange2;
double TreeNode2c::alpha = 0.0;


void TreeNode2c::cmpBnds(){
   Res xx1, yy1, xx2, yy2;

   if ((!findEnds(Res(winPair1.win1.ul_x, winPair1.win1.lr_x),
         Res(winPair1.win2.ul_x, winPair1.win2.lr_x),
         Res(szRange1.wmin, szRange1.wmax), &xx1)) ||
       (!findEnds(Res(winPair1.win1.ul_y, winPair1.win1.lr_y),
         Res(winPair1.win2.ul_y, winPair1.win2.lr_y),
         Res(szRange1.hmin, szRange1.hmax), &yy1)) ||
       (!findEnds(Res(winPair2.win1.ul_x, winPair2.win1.lr_x),
         Res(winPair2.win2.ul_x, winPair2.win2.lr_x),
         Res(szRange2.wmin, szRange2.wmax), &xx2)) ||
       (!findEnds(Res(winPair2.win1.ul_y, winPair2.win1.lr_y),
         Res(winPair2.win2.ul_y, winPair2.win2.lr_y),
         Res(szRange2.hmin, szRange2.hmax), &yy2))) {

       lb = INF;
       ub = INF;
       return;
   }

   Window ubWin1 = Window(xx1.x, yy1.x, xx1.y, yy1.y);
   Window ubWin2 = Window(xx2.x, yy2.x, xx2.y, yy2.y);

   if ((ubWin1.ul_x > ubWin1.lr_x) || (ubWin1.ul_y > ubWin1.lr_y) ||
       (ubWin2.ul_x > ubWin2.lr_x) || (ubWin2.ul_y > ubWin2.lr_y) ){
       bestBBoxes = WindowPair();
   } else bestBBoxes = WindowPair(ubWin1, ubWin2);

   int *h1_ub = ubWin1.getHist(integralIms1, nHistBin);
   int *h2_ub = ubWin2.getHist(integralIms2, nHistBin);
   ub = cmpEnergy5(h1_ub, h2_ub, nHistBin, alpha);

   Window smallestWin1 = winPair1.getSmallestWin();
   Window largestWin1  = winPair1.getLargestWin();

   Window smallestWin2 = winPair2.getSmallestWin();
   Window largestWin2  = winPair2.getLargestWin();

   int *h1_min = smallestWin1.getHist(integralIms1, nHistBin);
   int *h1_max = largestWin1.getHist(integralIms1, nHistBin);
   int *h2_min = smallestWin2.getHist(integralIms2, nHistBin);
   int *h2_max = largestWin2.getHist(integralIms2, nHistBin);

   double u12 = 0.0; //upper bound of h1^Th2
   double l1 = 0.0, l2 = 0.0; //lower bounds of |h1|^2, |h2|^2

   for (int i=0; i < nHistBin; i++)  {
      u12 += h1_max[i]*h2_max[i];
      l1 += h1_min[i]*h1_min[i];
      l2 += h2_min[i]*h2_min[i];
   }

   double l1122 = sqrt(l1*l2);
   if (l1122 > u12) lb = -u12/(l1122 + alpha);
   else lb = -u12/(u12 + alpha);

   destroyVector<int>(h1_ub, nHistBin);
   destroyVector<int>(h2_ub, nHistBin);
   destroyVector<int>(h1_min, nHistBin);
   destroyVector<int>(h1_max, nHistBin);
   destroyVector<int>(h2_min, nHistBin);
   destroyVector<int>(h2_max, nHistBin);
}




WindowPair findBoxes2c( int **kpAssign1, int nKeyPnt1, int imH1, int imW1, SizeRange szRange1,
      int **kpAssign2, int nKeyPnt2, int imH2, int imW2, SizeRange szRange2,
      int nHistBin, double alpha, double tolFac){

   int ***integralIms1 = cmpIntegralIms1(kpAssign1, nKeyPnt1, imH1, imW1, nHistBin);
   int ***integralIms2 = cmpIntegralIms1(kpAssign2, nKeyPnt2, imH2, imW2, nHistBin);
   TreeNode2c::integralIms1 = integralIms1;
   TreeNode2c::integralIms2 = integralIms2;
   TreeNode2c::nHistBin = nHistBin;

   TreeNode2c::szRange1 = szRange1;
   TreeNode2c::szRange2 = szRange2;
   TreeNode2c::alpha = alpha;

   Window biggestWin1(0,0, imW1-1, imH1-1);
   Window biggestWin2(0,0, imW2-1, imH2-1);

   TreeNode2c rootNode(WindowPair(biggestWin1, biggestWin1),
                       WindowPair(biggestWin2, biggestWin2));

   TreeNode2c bestNode = m_branch_bound<TreeNode2c>(rootNode, tolFac);

   destroyArrayThree<int>(integralIms1, imH1, imW1, nHistBin);
   destroyArrayThree<int>(integralIms2, imH2, imW2, nHistBin);
   return bestNode.bestBBoxes;
}
